import 'package:fis_lib_report/converts/event_type.dart';
import 'package:fis_lib_report/report/element_tag.dart';
import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/inputImageList.dart';
import 'package:fis_lib_report/report/interfaces/block_element.dart';
import 'package:fis_lib_report/report/paragraph.dart';
import 'package:fis_lib_report/report/report_template_document.dart';
import 'package:fis_lib_report/report/rt_page_size.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_lib_report/report_info/block_element_info_interface.dart';
import 'package:fis_lib_report/report_info/element_info.dart';
import 'package:fis_lib_report/report_info/element_tag_names.dart';
import 'package:fis_lib_report/report_info/input_image_list_info.dart';
import 'package:fis_lib_report/report_info/input_text_info.dart';
import 'package:fis_lib_report/report_info/paragraph_info.dart';
import 'package:fis_lib_report/report_info/report_event_args.dart';
import 'package:fis_lib_report/report_info/rt_table_info.dart';
import 'package:fis_lib_report/report_info/single_selected_info.dart';

class ReportBaseInfo {
  List<IBlockElementInfo> blocks = [];
  List<IBlockElementInfo> headers = [];
  List<IBlockElementInfo> footers = [];

  /// Gets the author of the report template.
  String? author;

  /// Gets or sets the value to indicate whether the report template is created by user.
  bool? isCustom;

  /// Gets the report template name.
  String? name;

  /// Gets the custom input element tags of the report template.
  Map<String, ElementTag>? tags;

  /// Gets or sets the update time.
  DateTime? updateTime;

  ///version
  String? version;

  /// Gets the id.
  String? id;

  //报告页大小
  RTPageSize? pageSize;

//报告者
  String? reporter;

  //报告病人名
  String? patientName;

  ///病人Id
  String? patinentId;

  ///病人年龄
  String? patientAge;

  ///病人性别
  String? patientSex;

  ///报告日期
  DateTime? reportDate;

  ///报告模板
  ReportTemplateDocument? reportTemplate;

  ///重载
  FEventHandler<ReportEventArgs> onReload = FEventHandler();

  ///重载完成时触发
  FEventHandler onReloadFinsh = FEventHandler();

  ///关闭页面，释放监听
  FEventHandler onClose = FEventHandler();

  ///初始化ReportInfo
  void init(
    ReportTemplateDocument reportTemplate,
    DateTime reportDate,
    String reporter, {
    String? patientName,
    String? patinentId,
    String? patientAge,
    String? patientSex,
  }) {
    try {
      headers = [];
      blocks = [];
      footers = [];
      this.reportTemplate = reportTemplate;
      this.reportDate = reportDate;
      this.reporter = reporter;
      isCustom = reportTemplate.isCustom;
      name = reportTemplate.name;
      updateTime = reportTemplate.updateTime;
      version = reportTemplate.version;
      id = reportTemplate.id;
      for (var element in reportTemplate.header!) {
        headers.add(_initBlockElement(element));
      }
      for (var element in reportTemplate.blocks!) {
        blocks.add(_initBlockElement(element));
      }
      for (var element in reportTemplate.footer!) {
        footers.add(_initBlockElement(element));
      }
      pageSize = reportTemplate.pageSize;
      final nameInputInfo = getElementInfoByTagName(TagNames.PATIENT_NAME_TAG);
      if (patientName != null &&
          patientName.isNotEmpty &&
          nameInputInfo != null) {
        final inputInfo = nameInputInfo as InputTextInfo;
        inputInfo.text = patientName;
      }
      final ageInputInfo = getElementInfoByTagName(TagNames.PATIENT_AGE_TAG);
      if (patientAge != null && patientAge.isNotEmpty && ageInputInfo != null) {
        final inputInfo = ageInputInfo as InputTextInfo;
        inputInfo.text = patientAge;
      }
      final idInputInfo = getElementInfoByTagName(TagNames.PATIENT_ID_TAG);
      if (patinentId != null && patinentId.isNotEmpty && idInputInfo != null) {
        final inputInfo = idInputInfo as InputTextInfo;
        inputInfo.text = patinentId;
      }
      final sexSelectedInfo = getElementInfoByTagName(TagNames.PATIENT_SEX_TAG);
      if (patientSex != null &&
          patientSex.isNotEmpty &&
          sexSelectedInfo != null) {
        final selectedInfo = sexSelectedInfo as SingleSelectedInfo;
        selectedInfo.selectedItem = patientSex;
      }
    } catch (e) {
      print(e);
    }
  }

  ///跟据TagName获取Elementnt
  ElementInfo? getElementInfoByTagName(String tagName) {
    try {
      for (var e in headers) {
        if (e.tag != null && e.tag!.name == tagName) {
          return e;
        }
        final elementInfo = _getBaseElementInfoByTagName(e, tagName);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in blocks) {
        if (e.tag != null && e.tag!.name == tagName) {
          return e;
        }
        final elementInfo = _getBaseElementInfoByTagName(e, tagName);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in footers) {
        if (e.tag != null && e.tag!.name == tagName) {
          return e;
        }
        final elementInfo = _getBaseElementInfoByTagName(e, tagName);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
    } catch (e) {
      print(e);
    }
    return null;
  }

  ///获取基础组件信息
  ElementInfo? _getBaseElementInfoByTagName(
      IBlockElementInfo block, String tagName) {
    ElementInfo? result;
    final type = block.elementType;
    if (type!.name == ElementType.paragraph.name) {
      final paragraph = block as ParagraphInfo;
      for (var el in paragraph.elementInfos!) {
        if (el.tag != null && el.tag!.name == tagName) {
          result = el;
          break;
        }
      }
    }
    if (result == null) {
      if (type.name == ElementType.rtTable.name) {
        final table = block as RTTableInfo;
        final cells = table.cells!;
        for (var cell in cells.values) {
          for (var b in cell.blocks!) {
            final info = _getBaseElementInfoByTagName(b, tagName);
            if (info != null) {
              result = info;
              break;
            }
          }
        }
      }
    }

    return result;
  }

  IBlockElementInfo _initBlockElement(IBlockElement element) {
    final _type = element.elementType!;
    if (_type.name == ElementType.rtTable.name) {
      final table = element as RTTable;
      return RTTableInfo.fromElement(table);
    } else if (_type.name == ElementType.paragraph.name) {
      final paragraph = element as Paragraph;
      return ParagraphInfo.fromElement(paragraph);
    } else {
      final inputImageList = element as InputImageList;
      return InputImageListInfo.fromElement(inputImageList);
    }
  }

  ///跟据Id获取Elementnt
  ElementInfo? getElementInfoById(String id) {
    try {
      if (id.isEmpty) {
        return null;
      }
      for (var e in headers) {
        if (e.id == id) {
          return e;
        }
        final elementInfo = _getBaseElementInfoById(e, id);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in blocks) {
        if (e.id == id) {
          return e;
        }
        final elementInfo = _getBaseElementInfoById(e, id);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in footers) {
        if (e.id == id) {
          return e;
        }
        final elementInfo = _getBaseElementInfoById(e, id);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
    } catch (e) {
      print(e);
    }
    return null;
  }

  ///跟据Id获取基础组件信息
  ElementInfo? _getBaseElementInfoById(IBlockElementInfo block, String id) {
    ElementInfo? result;
    final type = block.elementType;
    if (type!.name == ElementType.paragraph.name) {
      final paragraph = block as ParagraphInfo;
      for (var el in paragraph.elementInfos!) {
        if (el.tag != null && el.id == id) {
          result = el;
          break;
        }
      }
    }
    if (result == null) {
      if (type.name == ElementType.rtTable.name) {
        final table = block as RTTableInfo;
        final cells = table.cells!;
        for (var cell in cells.values) {
          for (var b in cell.blocks!) {
            final info = _getBaseElementInfoById(b, id);
            if (info != null) {
              result = info;
              break;
            }
          }
        }
      }
    }

    return result;
  }
}
